<?php

namespace BS\RealTimeChat\Admin\Controller;

use BS\RealTimeChat\Entity\Command;
use XF\Mvc\ParameterBag;
use XF\Mvc\FormAction;
use XF\Admin\Controller\AbstractController;

class Chat extends AbstractController
{
    protected function preDispatchController($action, ParameterBag $params)
    {
        if (preg_match('/^(commandadd)/i', $action))
        {
            $this->assertDebugMode();
            $this->assertDevelopmentMode();

            if (! $this->options()->rtcDevelopment)
            {
                throw $this->exception($this->noPermission(
                    \XF::phrase('rtc_this_page_is_only_available_when_development_mode_is_enabled')
                ));
            }
        }
    }

    public function actionIndex()
    {
        return $this->view('BS\RealTimeChat:Index', 'real_time_chat');
    }

    public function actionCommand()
    {
        $commands = $this->getCommandRepo()->findCommandsForList();
        $total = $commands->total();

        $viewParams = [
            'commands' => $commands->fetch(),
            'total' => $total
        ];
        return $this->view('BS\RealTimeChat:Command\List', 'real_time_chat_command_list', $viewParams);
    }

    protected function commandAddEdit(\BS\RealTimeChat\Entity\Command $command)
    {
        $viewParams = [
            'command' => $command
        ];
        return $this->view('BS\RealTimeChat:Command\Edit', 'real_time_chat_command_edit', $viewParams);
    }

    public function actionCommandEdit(ParameterBag $params)
    {
        $command = $this->assertCommandExists($params->command_id);
        return $this->commandAddEdit($command);
    }

    public function actionCommandAdd()
    {
        /** @var Command $command */
        $command = $this->em()->create('BS\RealTimeChat:Command');
        return $this->commandAddEdit($command);
    }

    protected function commandSaveProcess(\BS\RealTimeChat\Entity\Command $command)
    {
        $form = $this->formAction();

        $input = $this->filter([
            'command_id' => 'str',
            'command_class' => 'str',
            'command_regex' => 'str',
            'active' => 'bool',
            'addon_id' => 'str'
        ]);

        $extraInput = $this->filter([
            'title' => 'str'
        ]);

        $debug = (\XF::$debugMode && \XF::$developmentMode);

        $form->validate(function(FormAction $form) use ($input, $extraInput, $debug, $command)
        {
            if ($command->isInsert() && $input['command_id'] == '')
            {
                $form->logError(\XF::phrase('please_enter_valid_value'), 'command_id');
            }

            if (@preg_match($input['command_regex'], '') === false)
            {
                $form->logError(\XF::phrase('please_enter_valid_value'), 'command_regex');
            }

            if ($debug && $extraInput['title'] == '')
            {
                $form->logError(\XF::phrase('please_enter_valid_value'), 'title');
            }
        });

        if ($debug)
        {
            $form->apply(function() use ($extraInput, $command)
            {
                $title = $command->getMasterTitlePhrase();
                $title->phrase_text = $extraInput['title'];
                $title->save();
            });
        }
        else
        {
            unset($input['command_class'], $input['command_regex']);
        }

        if (!$command->isInsert())
        {
            unset($input['command_id']);
        }

        $form->basicEntitySave($command, $input);

        return $form;
    }

    public function actionCommandSave(ParameterBag $params)
    {
        $this->assertPostOnly();

        if ($params->command_id)
        {
            $command = $this->assertCommandExists($params->command_id);
        }
        else
        {
            $command = $this->em()->create('BS\RealTimeChat:Command');
        }

        $this->commandSaveProcess($command)->run();

        return $this->redirect($this->buildLink('chat/commands')  . $this->buildLinkHash($command->command_id));
    }

    public function actionCommandDelete(ParameterBag $params)
    {
        $command = $this->assertCommandExists($params->command_id);

        $plugin = $this->plugin('XF:Delete');
        return $plugin->actionDelete(
            $command,
            $this->buildLink('chat/commands/delete', $command),
            $this->buildLink('chat/commands/edit', $command),
            $this->buildLink('chat/commands'),
            $command->title
        );
    }

    public function actionCommandToggle()
    {
        $plugin = $this->plugin('XF:Toggle');
        return $plugin->actionToggle('BS\RealTimeChat:Command');
    }

    /** @return \BS\RealTimeChat\Repository\Command */
    protected function getCommandRepo()
    {
        return $this->repository('BS\RealTimeChat:Command');
    }

    /** @return Command */
    protected function assertCommandExists($id, $with = null, $phraseKey = null)
    {
        return $this->assertRecordExists('BS\RealTimeChat:Command', $id, $with, $phraseKey);
    }
}