<?php

namespace BS\RealTimeChat\Entity;

use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;

/**
 * COLUMNS
 * @property string|null command_id
 * @property string command_regex
 * @property string command_class
 * @property bool active
 * @property string addon_id
 *
 * GETTERS
 * @property string title
 *
 * RELATIONS
 * @property \XF\Entity\Phrase MasterTitle
 * @property \XF\Entity\AddOn AddOn
 */

class Command extends Entity
{
    public function getMasterTitlePhrase()
    {
        $addonId = $this->addon_id;

        $phrase = $this->MasterTitle;
        if (!$phrase)
        {
            $phrase = $this->_em->create('XF:Phrase');
            $phrase->title = $this->_getDeferredValue(function() { return $this->getTitlePhraseName(); });
            $phrase->language_id = 0;
            $phrase->addon_id = $addonId;
        }

        return $phrase;
    }

    protected function _preSave()
    {
        if (strpos($this->command_class, ':') !== false)
        {
            $this->command_class = \XF::stringToClass($this->command_class, '%s\ChatCommand\%s');
        }

        if (!class_exists($this->command_class))
        {
            $this->error(\XF::phrase('invalid_class_x', ['class' => $this->command_class]), 'command_class');
        }
    }

    protected function _postSave()
    {
        $addonId = $this->addon_id;

        if ($this->isUpdate())
        {
            if ($this->isChanged('command_id') || $this->isChanged('addon_id'))
            {
                $titlePhrase = $this->getExistingRelation('MasterTitle');
                if ($titlePhrase)
                {
                    $titlePhrase->addon_id = $addonId;
                    $titlePhrase->title = $this->getTitlePhraseName();
                    $titlePhrase->save();
                }
            }
        }

        $this->rebuildCommandsCache();
    }

    protected function _postDelete()
    {
        $phrase = $this->MasterTitle;
        if ($phrase && $phrase->exists())
        {
            $phrase->delete();
        }

        $this->rebuildCommandsCache();
    }

    protected function rebuildCommandsCache()
    {
        \XF::runOnce('rebuildActiveBsRtcCommandsCache', function()
        {
            $this->getCommandRepo()->rebuildCommandsCache();
        });
    }

    public function getTitlePhraseName()
    {
        return 'rtc_command.' . $this->command_id;
    }

    public function getTitle()
    {
        if (!$this->MasterTitle)
        {
            return '';
        }
        
        return \XF::phrase($this->getTitlePhraseName());
    }

    public static function getStructure(Structure $structure)
    {
        $structure->table = 'xf_bs_chat_command';
        $structure->shortName = 'BS\RealTimeChat:Command';
        $structure->primaryKey = 'command_id';
        $structure->columns = [
            'command_id' => ['type' => self::BINARY, 'maxLength' => 25, 'match' => 'alphanumeric', 'required' => true],
            'command_regex' => ['type' => self::STR, 'required' => true],
            'command_class' => ['type' => self::STR, 'maxLength' => 100, 'required' => true],
            'active' => ['type' => self::BOOL, 'default' => true],
            'addon_id' => ['type' => self::BINARY, 'maxLength' => 50, 'default' => '']
        ];
        $structure->getters = [
            'title' => true
        ];
        $structure->behaviors = [
            'XF:DevOutputWritable' => []
        ];
        $structure->relations = [
            'MasterTitle' => [
                'entity' => 'XF:Phrase',
                'type' => self::TO_ONE,
                'conditions' => [
                    ['language_id', '=', 0],
                    ['title', '=', 'rtc_command.', '$command_id']
                ]
            ],
            'AddOn' => [
                'entity' => 'XF:AddOn',
                'type' => self::TO_ONE,
                'conditions' => 'addon_id',
                'primary' => false
            ]
        ];
        $structure->defaultWith = ['AddOn'];

        return $structure;
    }

    protected function getCommandRepo()
    {
        return $this->repository('BS\RealTimeChat:Command');
    }
}