<?php

namespace BS\RealTimeChat\Entity;

use BS\RealTimeChat\Listener;
use XF\Entity\User;
use XF\Mvc\Entity\Entity;
use XF\Mvc\Entity\Structure;
use XF\Template\Templater;

/**
 * COLUMNS
 * @property int|null message_id
 * @property int user_id
 * @property int to_user_id
 * @property int pm_user_id
 * @property int message_date
 * @property string message
 *
 * GETTERS
 * @property string message_body
 *
 * RELATIONS
 * @property User User
 * @property User ToUser
 * @property User PmUser
 */

class Message extends Entity
{
	public function forTemplate()
	{
	    /** @var Templater $templater */
		$templater = $this->app()->templater();
		$func = Listener::templaterFunc();

		return [
			'user_id' => $this->user_id,
			'message_id' => $this->message_id,
			'to_user_id' => $this->to_user_id,
			'pm_user_id' => $this->pm_user_id,
			'avatar' => $templater->$func('avatar', [$this->User, 'xxs']),
            'username' => $templater->$func('username_link', [$this->User, true, [
                'notooltip' => true,
                'data-xf-click' => 'chat-insert',
                'data-insert' => "/to @{$this->User->username}, ",
                'data-replace' => "/pm @{$this->User->username}, ",
                'data-regex' => '^(\/pm|\/to)\s@.[^\s]*,',
                'data-is-replacer' => 'true'
            ]]),
            'message' => $this->message_body,
            'message_date' => $templater->$func('date_dynamic', [$this->message_date])
		];
	}

	public function getMessageBody()
    {
        $func = Listener::templaterFunc();
        return $this->app()->templater()->$func('bb_code', [$this->app()->stringFormatter()->censorText($this->message), 'chat:message', $this]);
    }

	public function getMessageDate()
	{
		return round($this->message_date_ / 1000);
	}

	public function isTo()
	{
		if ($this->to_user_id && $this->to_user_id == \XF::visitor()->user_id)
		{
			return true;
		}

		return false;
	}

	public function isPm()
	{
		$visitor = \XF::visitor();
		
		if ($this->pm_user_id 
			&& ($this->pm_user_id == $visitor->user_id
			|| $this->user_id == $visitor->user_id))
		{
			return true;
		}

		return false;
	}

	public function canView()
	{
		if ($this->pm_user_id)
		{
			$visitor = \XF::visitor();

			if ($this->pm_user_id != $visitor->user_id && $this->user_id != $visitor->user_id)
			{
				return false;
			}
		}

		return true;
	}

	public function canEdit()
	{
		$visitor = \XF::visitor();

		if ($visitor->hasChatPermission('canEdit') && $this->user_id == $visitor->user_id)
		{
			return true;
		}

		return $visitor->hasChatPermission('canEditAny');
	}

	public function canDelete()
	{
		$visitor = \XF::visitor();

		if ($visitor->hasChatPermission('canDelete') && $this->user_id == $visitor->user_id)
		{
			return true;
		}

		return $visitor->hasChatPermission('canDeleteAny');
	}

	public function canReport(&$error = '', User $asUser = null)
	{
		$asUser = $asUser ?: \XF::visitor();
		return $asUser->canReport($error);
	}

	public static function getStructure(Structure $structure)
	{
		$structure->table = 'xf_bs_chat';
		$structure->shortName = 'BS\RealTimeChat:Message';
		$structure->primaryKey = 'message_id';
		$structure->contentType = 'chat_message';
		$structure->columns = [
			'message_id' => ['type' => self::UINT, 'autoIncrement' => true],
			'user_id' => ['type' => self::UINT, 'required' => true, 'default' => \XF::visitor()->user_id],
			'message' => ['type' => self::STR, 'required' => true],
			'message_date' => ['type' => self::UINT, 'default' => round(microtime(true) * 1000), 'max' => pow(2, 63)-2],
			'to_user_id' => ['type' => self::UINT, 'default' => 0],
			'pm_user_id' => ['type' => self::UINT, 'default' => 0]
		];
		$structure->getters = [
			'message_date' => true,
            'message_body' => true
		];
		$structure->behaviors = [
            'XF:Indexable' => [
                'checkForUpdates' => ['message', 'user_id']
            ]
        ];
		$structure->relations = [
			'User' => [
				'entity' => 'XF:User',
				'type' => self::TO_ONE,
				'conditions' => 'user_id',
				'primary' => true
			],
			'ToUser' => [
				'entity' => 'XF:User',
				'type' => self::TO_ONE,
				'conditions' => [['user_id', '=', '$to_user_id']],
				'primary' => true
			],
			'PmUser' => [
				'entity' => 'XF:User',
				'type' => self::TO_ONE,
				'conditions' => [['user_id', '=', '$pm_user_id']],
				'primary' => true
			]
		];

		return $structure;
	}
}